/*
 * Copyright (C) 2015 Y.Sugahara (moveccr)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

// デバッグ用診断ツール

// グローバル
public class gDiag
{
	public static bool global_trace = false;
	public static bool global_debug = false;
	public static bool global_warn  = false;

	public static void Puts(string s, string className = "")
	{
		TimeVal tv = TimeVal();
		DateTime dt = new DateTime.from_timeval_local(tv);
		var time = dt.format("%T") + ".%03d".printf((int)tv.tv_usec / 1000);
		if (className == "") {
			stderr.puts(@"[$(time)] $(s)\n");
		} else {
			stderr.puts(@"[$(time)] $(className) $(s)\n");
		}
	}

	public static void PutHex(string s, uchar[] d, string className = "")
	{
		Puts(s, className);
		int len = d.length;
		for (int i = 0; i < len; i++) {
			stderr.printf(" %02X", d[i]);
			if (i % 16 == 15 && i < len - 1) {
				stderr.putc('\n');
			}
		}
		stderr.putc('\n');
	}

	public static void Trace(string s)
	{
		if (global_trace) {
			Puts(s);
		}
	}

	public static void Debug(string s)
	{
		if (global_debug) {
			Puts(s);
		}
	}

	public static void Warn(string s)
	{
		if (global_warn) {
			Puts(s);
		}
	}

	public static void DebugHex(string s, uchar[] d)
	{
		if (global_debug) {
			PutHex(s, d);
		}
	}

	public static void PROGERR(string s)
	{
		Puts(s, "PROGERR!!");
	}
}

// デバッグ用診断ツール
public class Diag
{
	public bool opt_trace;
	public bool opt_debug;
	public bool opt_warn;
	public string ClassName;

	public Diag(string className)
	{
		ClassName = className;
		opt_trace = gDiag.global_trace;
		opt_debug = gDiag.global_debug;
		opt_warn = gDiag.global_warn;
	}

	// 出力するログレベルを指定する
	//  0: warn を出力する
	//  1: warn, debug を出力する
	//  2: warn, debug, trace を出力する
	public void SetLevel(int level)
	{
		opt_trace = false;
		opt_debug = false;
		opt_warn  = false;

		if (level >= 0)
			opt_warn = true;
		if (level >= 1)
			opt_debug = true;
		if (level >= 2)
			opt_trace = true;
	}

	public void Trace(string s)
	{
		if (opt_trace) {
			gDiag.Puts(s, ClassName);
		}
	}

	public void Debug(string s)
	{
		if (opt_debug) {
			gDiag.Puts(s, ClassName);
		}
	}

	public void Warn(string s)
	{
		if (opt_warn) {
			gDiag.Puts(s, ClassName);
		}
	}

	// 一部分を表示したいときはスライスを使う。
	public void DebugHex(string s, uchar[] d)
	{
		if (opt_debug) {
			gDiag.PutHex(s, d, ClassName);
		}
	}
}
